#ifndef _COMMON_
#define _COMMON_
#include <string>

#define PROGRAM_VERSION "1.0.1"
// LANG=ru_RU.UTF-8
#define LANG_RU "ru_RU.UTF-8"
#define DEFAULTADDRESS "127.0.0.1:4999"

/*
Show help screen.
*/
void show_help(bool isRulang);
/// @brief Отображает отладочную информацию
void ShowDebug(std::string url, std::string result, bool success, std::string decoded);

/// @breef Replace in string
std::string Replace(std::string source, std::string from, std::string out);

/// @breef Replace all entryes in string
std::string ReplaceAll(std::string str, const std::string &from, const std::string &to);

/// @breef Вывод сообщения о таймауте или пустой строке ответа
int NoResponse(std::string mgs);

#endif
